// *SPARK D-FUSER
// A project by *spark audio-visual
//
// 'DJ' controller styke RS232 Control for TV-One products
// Good for 1T-C2-750, others will need some extra work
//
// Copyright *spark audio-visual 2009-2010

#include "spk_tvone.h"

// #define DEBUG

// Luma and Chroma key values. Note these are totally specific to your source. Do not assume one laptop equals another.
#define lumaMinY 0
#define lumaMaxY 18
#define lumaMinU 128
#define lumaMaxU 129
#define lumaMinV 128
#define lumaMaxV 129


#define chromaMinY 41
#define chromaMaxY 42
#define chromaMinU 240
#define chromaMaxU 241
#define chromaMinV 109
#define chromaMaxV 110

// Input Pins
#define aXFadePin 0    // select the input pin for the xfade pot
#define aFadeOutPin 1  // select the input pin for the fade to black pot
#define bTapLeftPin 2
#define bTapRightPin 3
#define aDIPPin 2 // Temporary, to be replaced by menu

// Output Pins
#define statusLED 9

// Fade logic constants + variables
const int xFadeTolerance = 10;
const int fadeOutTolerance = 100;

int xFadePercent = 0;    
int lastXFadePercent = 0;
int fadeUpPercent = 100;
int lastfadeUpPercent = 100;

enum { blend, lumaKey, chromaKey } ;
int mode = blend;

// Tap button states
boolean tapLeftPrevious = false;
boolean tapRightPrevious = false;

// DIPs - Temporary, to be replaced by menu
int dipState = 0; 
unsigned long dipDebounce = 0;

TVOne tvOne;

boolean dipChanged (int analog) 
{
 // Empirically for v1 controller pcb
 // 1024 = all up
 // 412 = 1 down
 // 320 = 2 down
 // 0 = 3 down
 int newDIP; 
 newDIP = dipState;
 if (analog >= 1020) { newDIP = 0; dipDebounce = millis(); }
 else if (analog >= 410 && analog <= 414) { newDIP = 1; dipDebounce = millis(); }
 else if (analog >= 318 && analog <= 322) { newDIP = 2; dipDebounce = millis(); }
 else if (analog <= 4) { newDIP = 3; dipDebounce = millis(); }
 
 if ((newDIP != dipState) && (dipDebounce + 500 > millis() )) {dipState = newDIP; return true; }
 else return false;
}

int fadeCalc (int analog, int tolerance) 
{
  int pos;
  if (analog < tolerance) pos = 0;
  else if (analog > 1023 - tolerance) pos = 100;
  else pos = 100 * (float)(analog - tolerance) / (1023 - 2*tolerance);
  return pos;
} 

void signError() {
//  digitalWrite(statusLED, HIGH);
//  delay(500);
//  digitalWrite(statusLED, LOW);
}

void setModeToBlend()
{
  #ifdef DEBUG
    Serial.println();
    Serial.println("setModeToBlend");
    Serial.println();
  #endif
  
  mode = blend;
  
    tvOne.command(0, kA, kTV1FunctionAdjustKeyerEnable, false);
}

void setModeToLuma()
{
  #ifdef DEBUG
    Serial.println();
    Serial.println("setModeToLuma");
    Serial.println();
  #endif
  
  mode = lumaKey;
  
  tvOne.command(0, kA, kTV1FunctionAdjustKeyerMinY, lumaMinY); 
  tvOne.command(0, kA, kTV1FunctionAdjustKeyerMinU, lumaMinU); 
  tvOne.command(0, kA, kTV1FunctionAdjustKeyerMinV, lumaMinV); 
  tvOne.command(0, kA, kTV1FunctionAdjustKeyerMaxY, lumaMaxY); 
  tvOne.command(0, kA, kTV1FunctionAdjustKeyerMaxU, lumaMaxU); 
  tvOne.command(0, kA, kTV1FunctionAdjustKeyerMaxV, lumaMaxV); 
  
  tvOne.command(0, kA, kTV1FunctionAdjustKeyerEnable, true);
}

void setModeToChroma()
{
  #ifdef DEBUG
    Serial.println();
    Serial.println("setModeToChroma");
    Serial.println();
  #endif
  
  mode = chromaKey;
  
    delay(20);  
    tvOne.command(0, kA, kTV1FunctionAdjustKeyerMinY, chromaMinY); delay(50);
    tvOne.command(0, kA, kTV1FunctionAdjustKeyerMinU, chromaMinU); delay(50);
    tvOne.command(0, kA, kTV1FunctionAdjustKeyerMinV, chromaMinV); delay(50);
    tvOne.command(0, kA, kTV1FunctionAdjustKeyerMaxY, chromaMaxY); delay(50);
    tvOne.command(0, kA, kTV1FunctionAdjustKeyerMaxU, chromaMaxU); delay(50);
    tvOne.command(0, kA, kTV1FunctionAdjustKeyerMaxV, chromaMaxV); delay(50);
  
    tvOne.command(0, kA, kTV1FunctionAdjustKeyerEnable, true);
}

void toggleMode(int modePressed)
{
  if (mode == blend)
  {
    if (modePressed == lumaKey)
      setModeToLuma();
    if (modePressed == chromaKey)
      setModeToChroma(); 
  }
  else if (mode == lumaKey)
  {
    if (modePressed == lumaKey)
      setModeToBlend();
    if (modePressed == chromaKey)
      setModeToChroma();
  }
  else if (mode == chromaKey)
  {
    if (modePressed == chromaKey)
      setModeToBlend();
    if (modePressed == lumaKey)
      setModeToLuma(); 
  }
}

void setup() 
{
  // init LEDs
  digitalWrite(statusLED, LOW);
  
  // FIXME: This should be called in the spk-tvone constructor, but only seems to work here
  Serial.begin(57600);
  
  tvOne.setHDCPOff();
  
  // ensure both windows are on-screen and black
//  tvOne.command(0, kA, kMaxFadeLevel, 0); 
//  tvOne.command(0, kB, kMaxFadeLevel, 0);

  digitalWrite(statusLED, HIGH);
} 

void loop() 
{
  
  if (dipChanged(analogRead(aDIPPin)))
  {
    //Serial.println(dipState, DEC);
    int32_t resolutionIndex;
    int32_t EDIDIndex;
    
    switch (dipState) {
      case 0: resolutionIndex = kTV1ResolutionVGA;              EDIDIndex = 1; break; 
      case 1: resolutionIndex = kTV1ResolutionSVGA;             EDIDIndex = 2; break; 
      case 2: resolutionIndex = kTV1ResolutionDualHeadSVGAp60;  EDIDIndex = 3; break; 
      case 3: resolutionIndex = kTV1ResolutionTripleHeadVGAp60; EDIDIndex = 4; break; 
      default: resolutionIndex = kTV1ResolutionSVGA;            EDIDIndex = 2; break; 
    }
    tvOne.command(0, kA, kOutputRes, resolutionIndex);
    // tvOne.command(kRGB1, kA, kEDIDToUse, EDIDIndex);
    // tvOne.command(kRGB2, kA, kEDIDToUse, EDIDIndex);
  }
  
  boolean updateFade = false;
  
  // Get new states of tap buttons, remembering at end of loop() assign these current values to the previous variables
  boolean tapLeft = (digitalRead(bTapLeftPin) == LOW) ? true : false;
  boolean tapRight = (digitalRead(bTapRightPin) == LOW) ? true : false;
  
//  #ifdef DEBUG
//    if (tapLeft) Serial.print("tapLeft ");
//    if (tapRight) Serial.print("tapRight ");
//  #endif
  
  // TEMP CHANGE OF TAPS TO TOGGLE LUMAKEY AND CHROMAKEY ON+OFF
  // When a tap is depressed, we can ignore any move of the crossfader but not fade to black
  if (tapLeft || tapRight) 
  {
    // If both are pressed, which was not pressed in the last loop?
    if (tapLeft && tapRight) {
      if (!tapLeftPrevious)
      {
        toggleMode(lumaKey);
        // analogWrite(statusLED, (float)xFadePercent*2.55);
      } 
      if (!tapRightPrevious)
      {
        toggleMode(chromaKey);
        // analogWrite(statusLED, (float)xFadePercent*2.55);
      }       
    }
    // If just one is pressed, is this it going high or the other going low?
    else if (tapLeft && (!tapLeftPrevious || tapRightPrevious)) 
    {
      toggleMode(lumaKey);
      // analogWrite(statusLED, (float)xFadePercent*2.55);
    }
    else if (tapRight && (!tapRightPrevious || tapLeftPrevious))
    {
      toggleMode(chromaKey);
      // analogWrite(statusLED, (float)xFadePercent*2.55);
    }    
  }  
  
   
  xFadePercent = fadeCalc(analogRead(aXFadePin), xFadeTolerance);
  // If the xFader has moved, or we returning from a tap, then update xFadePercent
  if ((xFadePercent != lastXFadePercent) || (tapLeftPrevious && !tapLeft) || (tapRightPrevious && !tapRight)) {
    lastXFadePercent = xFadePercent;
    updateFade = true;
    // analogWrite(statusLED, (float)xFadePercent*2.55);
  }
  
  fadeUpPercent = fadeCalc(analogRead(aFadeOutPin), fadeOutTolerance);
  if (fadeUpPercent != lastfadeUpPercent) {
    lastfadeUpPercent = fadeUpPercent;
    updateFade = true;
    // analogWrite(statusLED, (float)fadeUpPercent*2.55);
  }

  // FIXME: Really, we should have B at 100% and A fading in over that, with fade to black implemented as a fade in black layer on top of that correct mix.
  // There is no way to implement that though, and the alphas get messy, so this is the only way (afaik).
  if (updateFade)
  {
    switch (mode) {
    case blend:
      // Crossfade fades between A and B
    if (!tvOne.command(0, kA, kMaxFadeLevel, (xFadePercent*fadeUpPercent)/100)) signError(); 
    if (!tvOne.command(0, kB, kMaxFadeLevel, ((100-xFadePercent)*fadeUpPercent)/100)) signError();
      break;
    case lumaKey:
    case chromaKey:
      // Crossfader fades A in over a full B
    if (!tvOne.command(0, kA, kMaxFadeLevel, (xFadePercent*fadeUpPercent)/100)) signError(); 
    if (!tvOne.command(0, kB, kMaxFadeLevel, fadeUpPercent)) signError();
    }  
  }
  
  #ifdef DEBUG
    if (updateFade)
    {
      Serial.println();
      Serial.print("mode = ");
      Serial.print(mode);
      Serial.print("   fadeUpPercent = ");
      Serial.print(fadeUpPercent);
      Serial.print("   xFadePercent = ");
      Serial.print(xFadePercent);
      Serial.print("   A level = ");
      Serial.print((xFadePercent*fadeUpPercent)/100);
      Serial.print("   B level = ");
      Serial.print(((100-xFadePercent)*fadeUpPercent)/100);
      Serial.println();
      Serial.println();
    }
  #endif
  
  
  tapLeftPrevious = tapLeft;
  tapRightPrevious = tapRight;
}
